import json
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Dict, Optional, List

import requests

from sziszapangma.integration.record_id_iterator import RecordIdIterator
from sziszapangma.integration.record_path_provider import RecordPathProvider


class AsrProcessor(ABC):
    @abstractmethod
    def call_recognise(self, file_path: str) -> Dict[str, Any]:
        """
        Currently most important is field `transcript` with list of transcript
        words.
        """
        pass


class AsrWebClient(AsrProcessor):
    _url: str
    _auth_token: Optional[str]

    def __init__(self, url: str, auth_token: Optional[str]):
        super(AsrWebClient, self).__init__()
        self._url = url
        self._auth_token = auth_token

    def call_recognise(self, file_path: str) -> Dict[str, Any]:
        files = {"file": open(file_path, "rb")}
        headers = (
            dict({"Authorization": f"Bearer {self._auth_token}"})
            if self._auth_token is not None
            else dict()
        )
        res = requests.post(self._url, files=files, headers=headers, timeout=600)
        json_response = res.json()
        print(f"asr processing result {json_response}")
        return json_response


class AsrPathCacheClient(AsrProcessor):
    cache_path: str
    path_to_id: Dict[str, str]

    def __init__(
        self,
        cache_path: str,
        record_iterator: RecordIdIterator,
        record_path_provider: RecordPathProvider,
    ):
        super(AsrPathCacheClient, self).__init__()
        self._cache_path = cache_path
        self.path_to_id = {
            record_path_provider.get_path(it): it for it in record_iterator.get_all_records()
        }

    def call_recognise(self, file_path: str) -> Dict[str, Any]:
        path = Path(self._cache_path).joinpath(f"{self.path_to_id[file_path]}.json")
        print(f'cache path {path}')
        if not path.exists():
            raise Exception(f"path not exists {path}")
        return json.load(open(path, "r"))


class MultipleSourcesAsrProcessor(AsrProcessor):
    processors: List[AsrProcessor]

    def __init__(self, processors: List[AsrProcessor]):
        self.processors = processors

    def call_recognise(self, file_path: str) -> Dict[str, Any]:
        value = None
        for it in self.processors:
            if value is None:
                try:
                    value = it.call_recognise(file_path)
                except Exception as e:
                    print(e)
        if value is None:
            raise Exception("All processors can not process record")
        else:
            print(value)
            return value
